/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "winner.h"

void pokerWinnerGetFullHand(poker_t *poker,pokerplayer_t *player,card_t *cards){
  uint8_t i;

  // Add the dealer hand
  for(i = 0; i < poker->cardsFacing; i++) {
    cards[i] = poker->cards[i];
  }

  // Add the player hand
  for(i = 0; i < player->cardCount; i++) {
    cards[i+poker->cardsFacing] = player->cards[i];
  }

  // Sort by card value
  cardHandSort(cards, poker->cardsFacing+player->cardCount);
}

winning_t pokerWinnerGetStatus(poker_t *poker, pokerplayer_t *player) {
  winning_t winning;
  uint8_t i, j, l;
  int32_t index;
  card_t card;
  uint8_t number, suit;

  // Get the full poker hand (should be a 7 card hand)
  winning.size = poker->cardsFacing + player->cardCount;
  pokerWinnerGetFullHand(poker, player, winning.full);

  // Reset the winning status.
  winning.count = 0;
  winning.type = 0x00;
  for(i = 0; i < POKER_WINNING_SET_SIZE; i++) winning.set[i] = -1;

  // TESTING HAND
  winning.full[0] = CARD_CLUBS_ACE;
  winning.full[1] = CARD_CLUBS_TWO;
  winning.full[2] = CARD_CLUBS_THREE;
  winning.full[3] = CARD_SPADES_FOUR;
  winning.full[4] = CARD_CLUBS_FOUR;
  winning.full[5] = CARD_CLUBS_FIVE;
  winning.full[6] = CARD_CLUBS_JACK;
  
  //////////////////////// Now look for the winning set ////////////////////////

  // Royal / Straight Flush
  for(i = 0; i < winning.size; i++) {
    card = winning.full[i];
    number = cardGetNumber(card);
    if(number < CARD_FIVE) continue;

    suit = cardGetSuit(card);
    winning.count = 1;

    // Now look for the matching cards (Reverse order to order from A to 10)
    for(j = 1; j <= 4; j++) {
      l = card == CARD_FIVE && j == 4 ? CARD_ACE : number - j;//Ace low.
      index = cardContains(winning.full, winning.size, cardGet(l, suit));
      if(index == -1) break;
      winning.set[j] = index;
      winning.count++;
    }

    // Check if has all necessary cards.
    if(winning.count < POKER_WINNING_SET_SIZE) continue;

    // Add self to array
    winning.set[0] = card;
    winning.type = (
      number == CARD_ACE ? POKER_WINNING_TYPE_ROYAL_FLUSH : 
      POKER_WINNING_TYPE_STRAIGHT_FLUSH
    );
    if(winning.type == POKER_WINNING_TYPE_ROYAL_FLUSH) {
      printf("Royal Flush\n");
    } else {
      printf("Straight Flush\n");
    }
    return winning;
  }

  // High card (worst)

  return winning;
}

void pokerWinnerInit(poker_t *poker) {
  uint8_t winners[POKER_PLAYER_COUNT];
  uint8_t winnerCount = 0;

  poker->round = POKER_ROUND_WINNER;
  printf("Winner Winner Chicken Dinner\n");

  // Kill me
  for(uint8_t i = 0; i < POKER_PLAYER_COUNT; i++) {
    pokerplayer_t *player = poker->players + i;
    if(!pokerPlayerIsAlive(player)) continue;

    // Get the players' full hand
    pokerWinnerGetStatus(poker, player);
  }
}